import os
import os.path
import glob
import py2exe

manifest_template = '''
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
<assemblyIdentity
    version="0.64.1.0"
    processorArchitecture="x86"
    name="%(prog)s"
    type="win32"
/>
<description>%(prog)s Graphical User Interface</description>
<dependency>
    <dependentAssembly>
        <assemblyIdentity
            type="win32"
            name="Microsoft.Windows.Common-Controls"
            version="6.0.0.0"
            processorArchitecture="X86"
            publicKeyToken="6595b64144ccf1df"
            language="*"
        />
    </dependentAssembly>
</dependency>
</assembly>
'''

RT_MANIFEST = 24

################################################################
# arguments for the setup() call


class InnoScript:
    def __init__(self,
                 name,
                 lib_dir,
                 dist_dir,
                 console_exe_files = [],
                 windows_exe_files = [],
                 lib_files = [],
                 version = "1.6.1"):
        self.lib_dir = lib_dir
        self.dist_dir = dist_dir
        if not self.dist_dir[-1] in "\\/":
            self.dist_dir += "\\"
        self.name = name
        self.version = version
        self.windows_exe_files = [self.chop(p) for p in windows_exe_files]
        self.console_exe_files = [self.chop(p) for p in console_exe_files]
        self.lib_files = [self.chop(p) for p in lib_files]

    def chop(self, pathname):
        assert pathname.startswith(self.dist_dir)
        return pathname[len(self.dist_dir):]

    def create(self, pathname=r"dist/tidybot.iss"):
        self.pathname = pathname
        ofi = self.file = open(pathname, "w")
        print >> ofi, "; WARNING: This script has been created by py2exe. Changes to this script"
        print >> ofi, "; will be overwritten the next time py2exe is run."
	print >> ofi, ";"
        print >> ofi, r"[Setup]"
        print >> ofi, r"AppName=%s" % self.name
        print >> ofi, r"AppVerName=%s %s" % (self.name, self.version)
        print >> ofi, r"AppPublisher=%s" % "Kronto Software"
        print >> ofi, r"AppPublisherURL=%s" % "http://www.kronto.org/"
        print >> ofi, r"AppSupportURL=%s" % "http://www.kronto.org/tidybot/"
        print >> ofi, r"AppUpdatesURL=%s" % "http://www.kronto.org/tidybot/"
        print >> ofi, r"AppVersion=%s" % self.version
	print >> ofi, "AppComments=Tidybot is a batch HTML checker."
	print >> ofi, r"AppContact=Leo Breebaart <leo@kronto.org>"
	print >> ofi, r"AppReadmeFile={app}\README.txt"
	print >> ofi, r"InfoAfterFile=NEWS.txt"
	print >> ofi, r"InfoBeforeFile=LICENSE.txt"

        print >> ofi, r"DefaultDirName={pf}\%s" % self.name
        print >> ofi, r"DefaultGroupName=%s" % self.name
        print >> ofi, r"OutputDir=."
        print >> ofi, r"OutputBaseFilename=%s-%s-Installer" % (self.name, self.version)
	print >> ofi, r"SetupIconFile=installer.ico"
#	print >> ofi, r"Compression=none"
	print >> ofi, r"Compression=lzma/max"
	print >> ofi, r"SolidCompression=yes"
	print >> ofi, r"VersionInfoDescription=Tidybot 1.6.1 Installer"
	print >> ofi, r"VersionInfoTextVersion=This is the installer for the Tidybot programs."
	print >> ofi, r"VersionInfoversion=1.1"
	print >> ofi, r"VersionInfoCopyright=(C)opyright 2005, Kronto Software."
	print >> ofi, r"WizardImageFile=installer-leftsplash.bmp"
	print >> ofi, r"WizardSmallImageFile=installer-toplogo.bmp"
        print >> ofi

        print >> ofi, r"[CustomMessages]"
        print >> ofi, r"CreateDesktopIcon=Create a &Desktop icon"
        print >> ofi

	print >> ofi, r"[Tasks]"
	print >> ofi, r'Name: "desktopicon"; Description: "{cm:CreateDesktopIcon}"; GroupDescription: "{cm:AdditionalIcons}"; Flags: unchecked'
	print >> ofi, r'Name: "quicklaunchicon"; Description: "{cm:CreateQuickLaunchIcon}"; GroupDescription: "{cm:AdditionalIcons}"; Flags: unchecked'
	print >> ofi

        print >> ofi, r"[Files]"
        for path in self.console_exe_files + self.windows_exe_files + self.lib_files:
            print >> ofi, r'Source: "%s"; DestDir: "{app}\%s"; Flags: ignoreversion' % (path, os.path.dirname(path))
        print >> ofi
        print >> ofi, r'Source: "README.txt"; DestDir: "{app}\%s"; Flags: ignoreversion isreadme' % (os.path.dirname(path))
	print >> ofi

	print >> ofi, r"[INI]"
	print >> ofi, r'Filename: "{app}\%s.url"; Section: "InternetShortcut"; Key: "URL"; String: "%s"' % (self.name, "http://www.kronto.org/tidybot/")
	print >> ofi

        print >> ofi, r"[Icons]"
#        print >> ofi, r'Name: "{group}\%s"; Filename: "{app}\%s"; IconFilename: {app}\tidybot.ico' % (self.name, self.console_exe_files[0])
        print >> ofi, r'Name: "{group}\Tidybot GUI"; Filename: "{app}\%s"; IconFilename: {app}\tidybot.ico' % self.windows_exe_files[0]
	print >> ofi, r'Name: "{group}\{cm:ProgramOnTheWeb, %s}"; Filename: "{app}\%s.url"' % (self.name, self.name)
        print >> ofi, r'Name: "{group}\%s README"; Filename: "{app}\README.txt"' % self.name
        print >> ofi, r'Name: "{group}\{cm:UninstallProgram, %s}"; Filename: "{uninstallexe}"' % self.name
        print >> ofi, r'Name: "{userdesktop}\Tidybot GUI"; Filename: "{app}\Tidybot GUI.exe"; Tasks: desktopicon; IconFilename: {app}\tidybot.ico'
        print >> ofi, r'Name: "{userappdata}\Microsoft\Internet Explorer\Quick Launch\Tidybot GUI"; Filename: "{app}\Tidybot GUI.exe"; Tasks: quicklaunchicon; IconFilename: {app}\tidybot.ico'
	print >> ofi

        print >> ofi, r"[Run]"
	print >> ofi, r'Filename: "{app}\Tidybot GUI"; Description: "Launch %s"; Flags: shellexec postinstall skipifsilent' % self.name
	print >> ofi

	print >> ofi, r"[UninstallDelete]"
	print >> ofi, r'Type: files; Name: "{app}\%s.url"' % (self.name)
	print >> ofi, r'Type: files; Name: "{app}\Tidybot GUI.exe"'
	print >> ofi, r'Type: dirifempty; Name: "{app}"'
	print >> ofi


    def compile(self):
        try:
            import ctypes
        except ImportError:
            try:
                import win32api
            except ImportError:
                import os
                os.startfile(self.pathname)
            else:
                print "Ok, using win32api."
                win32api.ShellExecute(0, "compile",
                                                self.pathname,
                                                None,
                                                None,
                                                0)
        else:
            print "Cool, you have ctypes installed."
            res = ctypes.windll.shell32.ShellExecuteA(0, "compile",
                                                      self.pathname,
                                                      None,
                                                      None,
                                                      0)
            if res < 32:
                raise RuntimeError, "ShellExecute failed, error %d" % res

################################################################

from py2exe.build_exe import py2exe

class build_installer(py2exe):
    # This class first builds the exe file(s), then creates a Windows installer.
    # You need InnoSetup for it.
    def run(self):
        # First, let py2exe do its work.
        py2exe.run(self)

        lib_dir = self.lib_dir
        dist_dir = self.dist_dir

        # create the Installer, using the files py2exe has created.
        script = InnoScript("Tidybot",
                            lib_dir,
                            dist_dir,
                            self.console_exe_files,
                            self.windows_exe_files,
                            self.lib_files)
        print "*** creating the inno setup script***"
        script.create()
        print "*** compiling the inno setup script***"
        script.compile()
        # Note: By default the final setup.exe will be in an Output subdirectory.

################################################################

tidybot_gui = dict(
    script = r"tidybot_gui\wx_tidybot.py",
    icon_resources = [(1, r"tidybot_gui\tidybot.ico")],
    other_resources = [(RT_MANIFEST, 1, manifest_template % dict(prog="Tidybot GUI"))],
    dest_base = r"Tidybot GUI")

tidybot = dict(
    script = r"tidybot\tidybot.py",
    dest_base = r"tidybot")

zipfile = r"lib\library.zip"

includes = ["encodings.ascii"]

options = {"py2exe": {"compressed": 1,
                      "optimize": 2,
		      "includes": includes}}

cmdclass = {"py2exe": build_installer}

windows = [ tidybot_gui ]

console = [ tidybot ]

data_files = [('', [r'c:\Python24\Lib\site-packages\tidy\cygtidy-0-99-0.dll']),
	      ('', [r'c:\Python24\msvcr71.dll']),
              ('templates', glob.glob(r'tidybot\templates\*.tmpl')),
              ('css', [r'tidybot\css\tidybot.css',
                       r'tidybot\css\tidybot-bg.png']),
              ('', [r'tidybot\NEWS.txt',
                    r'tidybot\TODO.txt',
                    r'tidybot\README.txt',
                    r'tidybot\INSTALL.txt',
                    r'tidybot\LICENSE.txt']),
              ('', [r'tidybot_gui\tidybot.ico']),
              ('example-sources', glob.glob(r'tidybot_gui\example-sources\*.html')),
              ('', [r'py2exe\installer.ico',
                    r'py2exe\installer-leftsplash.bmp',
                    r'py2exe\installer-toplogo.bmp'])
              ]


