#!python
"""Windows-specific part of the installation"""

import os, sys

def create_shortcut_safe(target,description,link_file,*args,**kw):
    """make a shortcut if it doesn't exist, and register its creation"""

    if not os.path.isfile(link_file):
        create_shortcut(target, description, link_file,*args,**kw)
        file_created(link_file)


def install():
    """Routine to be run by the win32 installer with the -install switch."""

    # Get some system constants
    prefix = sys.prefix
    python = prefix + r'\python.exe'
    # Lookup path to common startmenu ...
    ip_dir = get_special_folder_path('CSIDL_COMMON_PROGRAMS') + r'\Tidybot'

    # Some usability warnings at installation time.  I don't want them at the
    # top-level, so they don't appear if the user is uninstalling.
    try:
        import ctypes
    except ImportError:
        print ('ATTENTION: Tidybot will not work until you install the ctypes package.\n'
               'See <http://sourceforge.net/projects/ctypes>\n')

    try:
        import tidy
    except ImportError:
        print ('ATTENTION: Tidybot will not work until you install the uTidylib package.\n'
               'See <http://utidylib.sf.net>\n')

    try:
        import HTMLTemplate
    except ImportError:
        print ('ATTENTION: Tidybot will not work until you install the HTMLTemplate package.\n'
               'See <http://freespace.virgin.net/hamish.sanderson/htmltemplate.html>\n')

    # Create Tidybot entry ...
    if not os.path.isdir(ip_dir):
        os.mkdir(ip_dir)
        directory_created(ip_dir)

    # Create program shortcuts ...
#    f = ip_dir + r'\Tidybot.lnk'
#    a = prefix + r'\scripts\tidybot'
#    create_shortcut_safe(python,'Tidybot',f,a)

    # Create documentation shortcuts ...
    t = prefix + r'\Lib\site-packages\tidybot\README.txt'
    f = ip_dir + r'\Tidybot README.lnk'
    create_shortcut_safe(t, r'Tidybot README', f)
    t = prefix + r'\Lib\site-packages\tidybot\NEWS.txt'
    f = ip_dir + r'\Tidybot NEWS.lnk'
    create_shortcut_safe(t, r'Tidybot NEWS', f)

def remove():
    """Routine to be run by the win32 installer with the -remove switch."""
    pass

# main()
if len(sys.argv) > 1:
    if sys.argv[1] == '-install':
        install()
    elif sys.argv[1] == '-remove':
        remove()
    else:
        print "Script was called with option %s" % sys.argv[1]
