#!/usr/bin/env python2.4

""" To use this setup script to install Tidybot:

        cd tidybot-<version>
        python setup.py install

This script is based on Nicholas Chauvat's setup script for the
PyChecker package, and on the Fernando Perez's setup scripts for
IPython.

Therefore, parts of this file are:

Copyright (c) 2000-2001, MetaSlash Inc.
Copyright (c) 2001, 2002, 2003, 2004 Fernando Perez <fperez@colorado.edu>.
Copyright (c) 2001 Janko Hauser <jhauser@zscout.de> and Nathaniel Gray
<n8gray@caltech.edu>.
"""

import sys
import os
import tempfile
import shutil
import tidybot

def remove_file(file):
    try :
        os.unlink(file)
    except :
        pass

# BEFORE importing distutils, remove MANIFEST. distutils doesn't
# properly update it when the contents of directories change.
if os.path.exists('MANIFEST'): remove_file('MANIFEST')

from distutils.core import setup
from distutils.core import Distribution
from distutils import sysconfig
from distutils import __version__ as distutils_version

class Win32Distribution(Distribution):
    def __init__(self, attrs):
        import py2exe_setup
	attrs["options"] = py2exe_setup.options
        Distribution.__init__(self, attrs)
        self.name = 'Tidybot'
	self.ctypes_com_server = []
	self.com_server = []
	self.service = []
	self.isapi = []
        self.zipfile = py2exe_setup.zipfile
        self.data_files = py2exe_setup.data_files
	self.console = py2exe_setup.console
        self.windows = py2exe_setup.windows
        self.cmdclass = py2exe_setup.cmdclass


class UnixDistribution(Distribution):
    def __init__(self, attrs):
        Distribution.__init__(self, attrs)
        self.name = 'tidybot'


if __name__ == '__main__' :

    if distutils_version < "2.4.0":
        print "Error: the tidybot installer requires distutils v2.4.0 or later."
        print "(You have: distutils v%s)" % distutils_version
        sys.exit(1)

    if os.name == 'posix':
        os_name = 'posix'
    elif os.name in ['nt','dos']:
        os_name = 'windows'
    else:
        print "Error: unsupported operating system detected:", os.name
        sys.exit(1)


    # The '--prefix' option to setup.py is majorly broken. See
    # comments further down.
    prefix_install = False
    for arg in sys.argv:
        if '--prefix' in arg:
            prefix_install = True
            break

    # The function get_python_lib() is broken for the --prefix
    # option in all versions of distutils that I know of. If
    # --prefix was specified, the user will manually have to edit
    # the erroneous path that will be placed in the 'tidybot'
    # script.
    install_dir = sysconfig.get_python_lib() + os.sep + 'tidybot'
    tidybot_py = install_dir + os.sep + 'tidybot.py'
    py_exe = sys.executable

    script_suffix = ''
    script_str = '#!/bin/sh\n\n%s %s "$@"\n' % (py_exe, tidybot_py)

    if sys.platform == 'win32':
        arg_str = '%1 %2 %3 %4 %5 %6 %7 %8 %9'
        script_str = '%s %s %s\n' % (py_exe, tidybot_py, arg_str)
        script_suffix = '.bat'

    TIDYBOT_SCRIPT = 'tidybot' + script_suffix
    TIDYBOT_SCRIPT = os.path.join(tempfile.gettempdir(), TIDYBOT_SCRIPT)
    remove_file(TIDYBOT_SCRIPT)

    try:
        fp = open(TIDYBOT_SCRIPT, "w")
        fp.write(script_str)
        fp.close()
        if sys.platform != 'mac' :
            os.chmod(TIDYBOT_SCRIPT, 0755)
    except:
        print "Unable to create the tidybot script."
        raise

    install_dir = sysconfig.get_python_lib() + os.sep + 'tidybot_gui'
    tidybot_py = install_dir + os.sep + 'wx_tidybot.py'
    py_exe = sys.executable

    script_suffix = ''
    script_str = '#!/bin/sh\n\n%s %s "$@"\n' % (py_exe, tidybot_py)

    if sys.platform == 'win32':
        arg_str = '%1 %2 %3 %4 %5 %6 %7 %8 %9'
        script_str = '%s %s %s\n' % (py_exe, tidybot_py, arg_str)
        script_suffix = '.bat'

    TIDYBOT_GUI_SCRIPT = 'tidybot_gui' + script_suffix
    TIDYBOT_GUI_SCRIPT = os.path.join(tempfile.gettempdir(), TIDYBOT_GUI_SCRIPT)
    remove_file(TIDYBOT_GUI_SCRIPT)

    try:
        fp = open(TIDYBOT_GUI_SCRIPT, "w")
        fp.write(script_str)
        fp.close()
        if sys.platform != 'mac' :
            os.chmod(TIDYBOT_GUI_SCRIPT, 0755)
    except:
        print "Unable to create the tidybot_gui script."
        raise


    scriptfiles = [TIDYBOT_SCRIPT, TIDYBOT_GUI_SCRIPT]

    docfiles = ['README', 'NEWS', 'TODO', 'LICENSE', 'INSTALL']
    tidybot_packagefiles = ['templates/*.tmpl',
                            'css/tidybot.css',
                            'css/tidybot-bg.png'
                            ]
    tidybot_gui_packagefiles = ['tidybot.ico',
				'example-sources/*.html'
                                ]

    # Installing doc files is a minefield. What I *want* is to
    # install the top-lebel README file into the installed
    # tidybot/ directory. However, I cannot get the 'data_files'
    # keyword to setup() working reliably -- as soon as the
    # '--prefix' option to setup.py is used, things go haywire
    # again. Therefore I now kluge things as follows: the README
    # file exists at top level in the source distribution (as it
    # should), but I copy it here into the tidybot/ subdir, and
    # then subsequently install it as package_data. Also, for
    # Windows, the file gets a 'txt' extension appended.
    newdocfiles = []
    for docfile in docfiles:
        if sys.platform == 'win32':
            newdocfile = os.path.join('tidybot', docfile + '.txt')
            newdocfiles.append(docfile + '.txt')
            infile = open(docfile)
            lines = infile.read().splitlines()
            infile.close()
            doslines = "\r\n".join(lines) + "\r\n"
            outfile = open(newdocfile, "wb")
            outfile.write(doslines)
            outfile.close()
        else:
            newdocfile = os.path.join('tidybot', docfile)
            newdocfiles.append(docfile)
            shutil.copyfile(docfile, newdocfile)
    docfiles = newdocfiles
    tidybot_packagefiles += docfiles


    # Script to be run by the windows binary installer after the default setup
    # routine, to add shortcuts and similar windows-only things.  Windows
    # post-install scripts MUST reside in the scripts/ dir, otherwise distutils
    # doesn't find them.
    if 'bdist_wininst' in sys.argv:
        if len(sys.argv) > 2 and ('sdist' in sys.argv or 'bdist_rpm' in sys.argv):
            print >> sys.stderr,"Error: bdist_wininst must be run alone."
            sys.exit(1)
        scriptfiles.append('scripts/tidybot_win_post_install.py')

    if sys.platform == 'win32':
        distclass = Win32Distribution
        name = 'Tidybot'
    else:
        distclass = UnixDistribution
        name = 'tidybot'


    classifiers = """\
Development Status :: 5 - Production/Stable
Environment :: Console
Environment :: Win32 (MS Windows)
Environment :: X11 Applications :: Gnome
Intended Audience :: End Users/Desktop
Intended Audience :: System Administrators
License :: OSI Approved :: MIT License
Natural Language :: English
Operating System :: OS Independent
Programming Language :: Python
Topic :: Internet :: WWW/HTTP :: Site Management
Topic :: System :: Monitoring
Topic :: Utilities
"""

    setup(distclass=distclass,
          name = name,
#          version = tidybot.__version__,
          version = "1.6.1",
          description = 'A batch (X)HTML syntax-checker and report-generator.',
#	  comments = 'This is a development version (vx.y.z) of Tidybot.',
#	  company_name = 'Kronto Software',
          author = 'Leo Breebaart',
          author_email = 'leo@kronto.org',
          url = 'http://www.kronto.org/tidybot/',
          license = 'MIT License <http://www.opensource.org/licenses/mit-license.php>',
          platforms = ['any'],
          classifiers = filter(None, classifiers.split("\n")),
          long_description = """\
Tidybot traverses one or more source directories, and
runs all HTML files it finds through the TidyLib
checker, collecting all warnings and errors it
encounters and presenting them nicely in an XHTML
report. It is possible to specify files/directories to
exclude from the checks, warnings/errors to suppress in
the generated report, and 'key:value' options to pass
directly to the underlying TidyLib engine. The
generated report can use a custom CSS stylesheet.
Tidybot has both a command-line interface and a GUI.""",

          scripts = scriptfiles,
          packages = ['tidybot', 'tidybot_gui'],
          package_data = { 'tidybot': tidybot_packagefiles,
                           'tidybot_gui': tidybot_gui_packagefiles}
          )

    # cleanup old script file, it should have been installed
    remove_file(TIDYBOT_SCRIPT)
    remove_file(TIDYBOT_GUI_SCRIPT)

    # Cleanup renamed doc files, they should have been installed
    for f in docfiles:
        remove_file(os.path.join('tidybot', f))

    if prefix_install:
        print
        print ("ATTENTION: the --prefix option does not work correctly in this\n"
               "version of the installer. Tidybot will still work, but you\n"
               "will first have to edit the tidybot script and correct\n"
               "the path.\n")
