# wx_tidybot.py - a simple graphical frontend for Tidybot.
#

import sys
import os
import os.path
import platform
import imp
import inspect
import tempfile
import ConfigParser
import cPickle
import threading
import ctypes

try:
    import wx
    import wx.lib.filebrowsebutton as filebrowse
except ImportError:
    print "Error: Python module 'wx' not found"
    print "Get it from <http://www.wxpython.org/>"
    sys.exit(1)


try:
    import mongoose
    import mongoose.reporters
except ImportError:
    print "Error: Python module 'mongoose' not found"
    print "Get it from <http://themongoose.sourceforge.net/>"
    sys.exit(1)


import tidy
import tidybot
import tidybot_gui


class MyConfigParser(ConfigParser.SafeConfigParser):
    """ A ConfigParser that can handle list values. Originally
    created by Bengt Richter on comp.lang.python. """
    def __init__(self, defaults):
        ConfigParser.SafeConfigParser.__init__(self, defaults)

    def getlist(self, section, option, **keywords):
        s = self.get(section, option, **keywords)
	try:
            return cPickle.loads(s)
        except EOFError:
            raise ConfigParser.NoOptionError(option, section)

    def setlist(self, section, option, value):
	s = cPickle.dumps(value)
	self.set(section, option, s)


class Borg:
    _shared_state = {}
    def __init__(self):
        self.__dict__ = self._shared_state


# These two functions exist solely so that py2exe will be able to
# find the tidybot_gui data files correctly. (Relative pathnames
# do not work as expected if your execution context is a zipfile,
# as it is when running from a py2exe-generated executable.
def main_is_frozen():
    return (hasattr(sys, "frozen") or # new py2exe
            hasattr(sys, "importers") # old py2exe
            or imp.is_frozen("__main__")) # tools/freeze

def get_main_dir():
    if main_is_frozen():
        return os.path.dirname(sys.executable)
    return os.path.dirname(inspect.getabsfile(get_main_dir))


# Taken from the iPython 0.6.13 sources
#----------------------------------------------------------------------------
class HomeDirError(Exception):
    pass

def get_home_dir():
    """Return the closest possible equivalent to a 'home' directory.

    We first try $HOME.  Absent that, on NT it's $HOMEDRIVE\$HOMEPATH.

    Currently only Posix and NT are implemented, a HomeDirError exception is
    raised for all other OSes. """ #'

    try:
        return os.environ['HOME']
    except KeyError:
        if os.name == 'posix':
            raise HomeDirError,'undefined $HOME, IPython can not proceed.'
        elif os.name == 'nt':
            # For some strange reason, win9x returns 'nt' for os.name.
            try:
                return os.path.join(os.environ['HOMEDRIVE'],os.environ['HOMEPATH'])
            except:
                try:
                    # Use the registry to get the 'My Documents' folder.
                    import _winreg as wreg
                    key = wreg.OpenKey(wreg.HKEY_CURRENT_USER,
                                       "Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders")
                    homedir = wreg.QueryValueEx(key,'Personal')[0]
                    key.Close()
                    return homedir
                except:
                    return 'C:\\'
        elif os.name == 'dos':
            # Desperate, may do absurd things in classic MacOS. May work under DOS.
            return 'C:\\'
        else:
            raise HomeDirError,'support for your operating system not implemented.'
#---------------------------------------------------------------------------

# Based on code from the wxPython demo
class DirBrowseButtonWithHistory(filebrowse.FileBrowseButtonWithHistory):
    def __init__(self, parent, id = -1,
                 pos = wx.DefaultPosition, size = wx.DefaultSize,
                 style = wx.TAB_TRAVERSAL,
                 labelText = 'Select a directory:',
                 buttonText = 'Browse',
                 toolTip = 'Type directory name or browse to select',
                 dialogTitle = '',
                 startDirectory = '.',
#                 changeCallback = None,
                 dialogClass = wx.DirDialog,
                 newDirectory = False
                 ):
        filebrowse.FileBrowseButtonWithHistory.__init__(self, parent, id, pos, size, style,
                                                        labelText, buttonText, toolTip,
                                                        dialogTitle, startDirectory,
 #                                                       changeCallback = changeCallback
                                                        )
        self.dialogClass = dialogClass
        self.newDirectory = newDirectory

    def OnBrowse(self, ev = None):
        style=0

        if self.newDirectory:
          style|=wx.DD_NEW_DIR_BUTTON

        dialog = self.dialogClass(self,
                                  message = self.dialogTitle,
                                  defaultPath = self.startDirectory,
                                  style = style)

        if dialog.ShowModal() == wx.ID_OK:
            self.SetValue(dialog.GetPath())
        dialog.Destroy()


class GUI_History(Borg):

    def __init__(self):
        Borg.__init__(self)

        self.appdir = str(tidybot_gui.homedirectory.appdatadirectory())
        if sys.platform == 'win32':
            self.configfilename = os.path.join(self.appdir, "Tidybot", "gui_history.ini")
        else:
            self.configfilename = os.path.join(self.appdir, ".tidybot", "gui_history")


    def Read(self):

        package_dir = get_main_dir()

	temp_dir = tempfile.gettempdir()
	if sys.platform == 'win32':
	    import win32api
	    temp_dir = win32api.GetLongPathName(temp_dir)

        # default values for preferences.
        defaults = dict(show_advanced_options = False,
                        show_results = True,
                        xml_pass = False,
                        user_mode = "enduser",
                        version = tidybot.__version__,
                        source_directories = [os.path.join(package_dir, "example-sources")],
                        output_directories = [temp_dir],
                        include = '*.html,*.htm',
                        include_files = [],
                        exclude = '',
                        exclude_files = [],
                        suppress_files = [],
                        suppress = '',
                        tidy_options_files = [],
                        tidy_options_str = '',
                        output_directory = temp_dir,
                        include_file = '',
                        exclude_file = '',
                        suppress_file = '',
                        tidy_options_file = '',
                        stylesheet = '',
			mailto = '',
			note = '')

        for (key, value) in defaults.iteritems():
            setattr(self, key, value)

        # Initialise with default values.
        self.config = MyConfigParser(defaults)
        # Override with values read from config file.
        self.config.read([self.configfilename])

        if not self.config.has_section("General"):
            return

        # Check here if the config file belongs to an old version
        # of Tidybot. If so, don't even try to read anything
        # else.
        try:
            self.version = self.config.get("General", "version")
        except ConfigParser.NoOptionError:
            return

	if "1.5d" in self.version or "1.5b" in self.version:
	    self.version = tidybot.__version__
	    return

        try:
            self.show_advanced_options = self.config.getboolean("General", "show_advanced_options")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.show_results = self.config.getboolean("General", "show_results")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.xml_pass = self.config.getboolean("General", "xml_pass")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.user_mode = self.config.get("General", "user_mode")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.source_directories = self.config.getlist("General", "source_directories")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.output_directories = self.config.getlist("General", "output_directories")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.include_files = self.config.getlist("General", "include_files")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.include = self.config.get("General", "include")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.exclude_files = self.config.getlist("General", "exclude_files")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.exclude = self.config.get("General", "exclude")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.suppress_files = self.config.getlist("General", "suppress_files")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.suppress = self.config.get("General", "suppress")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.tidy_options_files = self.config.getlist("General", "tidy_options_files")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.tidy_options_str = self.config.get("General", "tidy_options_str")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.output_directory = self.config.get("General", "output_directory")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.include_file = self.config.get("General", "include_file")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.exclude_file = self.config.get("General", "exclude_file")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.suppress_file = self.config.get("General", "suppress_file")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.tidy_options_file = self.config.get("General", "tidy_options_file")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.stylesheet = self.config.get("General", "stylesheet")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.mailto = self.config.get("General", "mailto")
        except ConfigParser.NoOptionError:
            pass

        try:
            self.note = self.config.get("General", "note")
        except ConfigParser.NoOptionError:
            pass


    def Write(self):
        if self.configfilename:
            configdir = os.path.dirname(self.configfilename)
            if not os.path.exists(configdir):
                os.mkdir(configdir)
            configfile = file(self.configfilename, "w")

            if not self.config.has_section("General"):
                self.config.add_section("General")
            self.config.set("General", "show_advanced_options", str(self.show_advanced_options))
            self.config.set("General", "show_results", str(self.show_results))
            self.config.set("General", "xml_pass", str(self.xml_pass))
            self.config.set("General", "user_mode", self.user_mode)
            self.config.set("General", "version", self.version)
            self.config.setlist("General", "source_directories", self.source_directories)

            self.config.setlist("General", "output_directories", self.output_directories)
            self.config.setlist("General", "include_files", self.include_files)
            self.config.setlist("General", "exclude_files", self.exclude_files)
            self.config.setlist("General", "suppress_files", self.suppress_files)
            self.config.setlist("General", "tidy_options_files", self.tidy_options_files)

            self.config.set("General", "output_directory", self.output_directory)
            self.config.set("General", "include_file", self.include_file)
            self.config.set("General", "include", self.include)
            self.config.set("General", "exclude_file", self.exclude_file)
            self.config.set("General", "exclude", self.exclude)
            self.config.set("General", "suppress_file", self.suppress_file)
            self.config.set("General", "suppress", self.suppress)
            self.config.set("General", "tidy_options_file", self.tidy_options_file)
            self.config.set("General", "tidy_options_str", self.tidy_options_str)
            self.config.set("General", "stylesheet", self.stylesheet)
            self.config.set("General", "mailto", self.mailto)
            self.config.set("General", "note", self.note)

            self.config.write(configfile)
            configfile.close()
        return



class WindowHandler(Borg):

    locx, locy = 50, 50
    offset = 30

    def __init__(self):
	Borg.__init__(self)

    def GetSizeAndPosition(self, size, pos):
	if not size:
	    size=(800, 494)

	if not pos:
            xmax = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_X)
            ymax = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)
	    self.locx += self.offset
	    self.locy += self.offset
	    # test if win stays within window
	    if (size[0] + self.locx) > xmax - self.offset or (size[1] + self.locy) > ymax - self.offset:
	        self.locx=self.offset
	        self.locy=self.offset
	else:
	    self.locx=pos[0]
	    self.locy=pos[1]

	pos = (self.locx, self.locy)
	return size, pos




# Define notification event for thread completion
EVT_EXCEPTION_ID = wx.NewId()

def EVT_EXCEPTION(win, func):
    win.Connect(-1, -1, EVT_EXCEPTION_ID, func)

EVT_FINISHED_ID = wx.NewId()

def EVT_FINISHED(win, func):
    win.Connect(-1, -1, EVT_FINISHED_ID, func)

EVT_UPDATE_ID = wx.NewId()

def EVT_UPDATE(win, func):
    win.Connect(-1, -1, EVT_UPDATE_ID, func)

class FinishedEvent(wx.PyEvent):
    def __init__(self):
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_FINISHED_ID)

class ExceptionEvent(wx.PyEvent):
    def __init__(self, eeks):
        wx.PyEvent.__init__(self)
        self.eeks = eeks
        self.SetEventType(EVT_EXCEPTION_ID)

class UpdateEvent(wx.PyEvent):
    def __init__(self, count, max_count):
        wx.PyEvent.__init__(self)
        self.count = count
        self.max_count = max_count
        self.SetEventType(EVT_UPDATE_ID)


# Thread class that executes processing
class WorkerThread(threading.Thread):
    def __init__(self, notify_window, config):
        threading.Thread.__init__(self)
        self._config = config
        self._notify_window = notify_window
        self._want_abort = 0
        # This starts the thread running on creation, but you could
        # also make the GUI thread responsible for calling this
        self.start()


    def run(self):
        # This is the code executing in the new thread.
        def update_function(count, max_count):
            if count % 5 == 0 or count == max_count:
                wx.PostEvent(self._notify_window,UpdateEvent(count, max_count))

        try:
            tidybot.run_tidybot(self._config, update_function)
	except Exception, eeks:
            wx.PostEvent(self._notify_window, ExceptionEvent(eeks))
            return

        wx.PostEvent(self._notify_window, FinishedEvent())

    def abort(self):
        # Method for use by main thread to signal an abort
        self._want_abort = 1



class TidybotPanel(wx.Panel):

    def __init__(self, parent, ID):
        wx.Panel.__init__(self, parent, ID)

        self.history = GUI_History()
        self.CreateControls()
        self.InstallToolTips()
        self.CreateLayout()
        self.InstallEventListeners()

        self.worker = None


    def CreateControls(self):

	self.source_directories = wx.ListBox(self, -1,
					   size=(420, 50),
                                           name="Source Directories",
                                           style=wx.LB_EXTENDED)

	self.add_sourceButton = wx.Button(self, -1, "Add...")
	self.del_sourceButton = wx.Button(self, -1, "Remove")
        self.clear_sourceButton = wx.Button(self, -1, "Clear list")

        self.show_resultsCheckBox = wx.CheckBox(self, -1, "Display results in browser")
        self.xml_passCheckBox = wx.CheckBox(self, -1, "Also perform an XML check")

        self.output_directory_label = wx.StaticText(self, -1, "Output directory: ")
        self.output_directory = DirBrowseButtonWithHistory(self, -1,
                                                           size=(20,-1),
                                                           labelText="",
                                                           buttonText="Browse...",
                                                           dialogTitle="Select output directory:",
                                                           toolTip="Directory where Tidybot will place result files",
                                                           style = wx.DD_NEW_DIR_BUTTON,
                                                           newDirectory = True
                                                           )

        self.include_label = wx.StaticText(self, -1, "Patterns: ")
        self.include = wx.TextCtrl(self, -1, size=(20,-1))

        self.include_file_label = wx.StaticText(self, -1, "File: ")
        self.include_file = filebrowse.FileBrowseButtonWithHistory(self, -1,
                                                                   size=(20,-1),
                                                                   labelText="",
                                                                   buttonText="Browse...",
                                                                   toolTip="Include the filename patterns listed in this file"
                                                                   )

        self.exclude_label = wx.StaticText(self, -1, "Strings: ")
        self.exclude = wx.TextCtrl(self, -1, size=(20,-1))

        self.exclude_file_label = wx.StaticText(self, -1, "File: ")
        self.exclude_file = filebrowse.FileBrowseButtonWithHistory(self, -1,
                                                                   size=(20,-1),
                                                                   labelText="",
                                                                   buttonText="Browse...",
                                                                   toolTip="Exclude the strings listed in this file"
                                                                   )

        self.suppress_label = wx.StaticText(self, -1, "Strings: ")
        self.suppress = wx.TextCtrl(self, -1, size=(20,-1))

        self.suppress_file_label = wx.StaticText(self, -1, "File: ")
        self.suppress_file = filebrowse.FileBrowseButtonWithHistory(self, -1,
                                                                    size=(20,-1),
                                                                    labelText="",
                                                                    buttonText="Browse...",
                                                                    toolTip="Suppress the result lines listed in this file"
                                                                    )

        self.tidy_options_str_label = wx.StaticText(self, -1, "Strings: ")
        self.tidy_options_str = wx.TextCtrl(self, -1, size=(20,-1))

        self.tidy_options_file_label = wx.StaticText(self, -1, "File: ")
        self.tidy_options_file = filebrowse.FileBrowseButtonWithHistory(self, -1,
                                                                        size=(20,-1),
                                                                        labelText="",
                                                                        buttonText="Browse...",
                                                                        toolTip="Use Tidy configuration options listed in this file"
                                                                        )

	self.mailto_label = wx.StaticText(self, -1, "Mail report to: ")
	self.mailto = wx.TextCtrl(self, -1, size=(20, -1))

	self.note_label = wx.StaticText(self, -1, "Custom note: ")
	self.note = wx.TextCtrl(self, -1, size=(20, -1))

        self.stylesheet_label = wx.StaticText(self, -1, "Stylesheet: ")
        self.stylesheet = wx.TextCtrl(self, -1, size=(20,-1))

        self.applyButton = wx.Button(self, -1, "Run Tidybot")


	self.source_directories.AppendItems(self.history.source_directories)
        self.show_resultsCheckBox.SetValue(self.history.show_results)
        self.xml_passCheckBox.SetValue(self.history.xml_pass)
        self.include.SetValue(self.history.include)
        self.exclude.SetValue(self.history.exclude)
        self.suppress.SetValue(self.history.suppress)
        self.tidy_options_str.SetValue(self.history.tidy_options_str)
        self.stylesheet.SetValue(self.history.stylesheet)
	self.mailto.SetValue(self.history.mailto)
	self.note.SetValue(self.history.note)

		
	have_mailto = (self.mailto.GetValue().strip() != '')
	self.note_label.Enable(have_mailto)
        self.note.Enable(have_mailto)

        font = self.applyButton.GetFont()
        font.SetWeight(wx.BOLD)
        self.applyButton.SetFont(font)
	self.applyButton.SetDefault()
        self.applyButton.SetFocus()

        self.historyControls = [ self.include_file,
                                 self.exclude_file,
                                 self.suppress_file,
                                 self.tidy_options_file,
                                 self.output_directory
                                 ]
        self.InitHistoryLists()



    def InstallToolTips(self):

        self.source_directories.SetToolTip(wx.ToolTip("Directories containing HTML source files"))
        self.add_sourceButton.SetToolTip(wx.ToolTip("Add a source directory to the list"))
        self.del_sourceButton.SetToolTip(wx.ToolTip("Remove the selected source directory(ies) from the list"))
        self.clear_sourceButton.SetToolTip(wx.ToolTip("Clear the list of source directories"))
        self.show_resultsCheckBox.SetToolTip(wx.ToolTip("Display the Tidybot result pages in your WWW browser"))
        self.xml_passCheckBox.SetToolTip(wx.ToolTip("Run a simple XML check in addition to the XHTML check"))
        self.include.SetToolTip(wx.ToolTip("A comma-separated list of filename inclusion patterns"))
        self.exclude.SetToolTip(wx.ToolTip("A comma-separated list of filename exclusion strings"))
        self.suppress.SetToolTip(wx.ToolTip("A comma-separated list of error message suppression strings"))
        self.tidy_options_str.SetToolTip(wx.ToolTip("A comma-separated list of 'key:value' tidy options"))
        self.stylesheet.SetToolTip(wx.ToolTip("The URI for the CSS stylesheet to use in the resulting HTML files"))
        self.mailto.SetToolTip(wx.ToolTip("Mail a summary report to this address (Unix-only)"))
        self.note.SetToolTip(wx.ToolTip("Custom text to include in summary report"))
        self.applyButton.SetToolTip(wx.ToolTip("Run Tidybot on the files in the source directories"))



    def CreateLayout(self):

        self.mainsizer = wx.BoxSizer(wx.VERTICAL)

 	source_sizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Source Directories"), wx.HORIZONTAL)

  	horbox = wx.BoxSizer(wx.HORIZONTAL)
	buttonsizer = wx.BoxSizer(wx.VERTICAL)
        buttonsizer.Add(self.add_sourceButton, 0, wx.EXPAND)
	buttonsizer.Add((6,6))
        buttonsizer.Add(self.del_sourceButton, 0, wx.EXPAND)
	buttonsizer.Add((20,20))
	buttonsizer.Add(self.clear_sourceButton, 0, wx.EXPAND)
        horbox.Add(self.source_directories, 1, wx.EXPAND)
	horbox.Add((10,10))
	horbox.Add(buttonsizer, 0, wx.EXPAND)
	source_sizer.Add(horbox, 1, wx.EXPAND|wx.ALL, border=10)

 	options_sizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Options"), wx.HORIZONTAL)
        verbox = wx.BoxSizer(wx.VERTICAL)
        verbox.Add(self.show_resultsCheckBox, 0, wx.ALIGN_LEFT)
        verbox.Add((10,10))
        verbox.Add(self.xml_passCheckBox, 0, wx.ALIGN_LEFT)
        options_sizer.Add(verbox, 1, wx.EXPAND|wx.ALL, border=10)

        horbox = wx.BoxSizer(wx.HORIZONTAL)
	horbox.Add(source_sizer, 1, wx.EXPAND)
	horbox.Add((10,10))
        horbox.Add(options_sizer, 0, wx.EXPAND)

	self.mainsizer.Add(horbox, 1, wx.EXPAND|wx.BOTTOM, border=10)

        self.advanced_options_sizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Advanced Options"), wx.VERTICAL)

        verbox = wx.BoxSizer(wx.VERTICAL)

        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add(self.output_directory_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.output_directory, 3, wx.LEFT, border=-5)
 	horbox.Add((15,-1))
        horbox.Add(self.stylesheet_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.stylesheet, 2, wx.ALIGN_CENTER_VERTICAL)
        verbox.Add(horbox, 0, wx.EXPAND)

        static_horbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Include"), wx.HORIZONTAL)
        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add(self.include_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.include, 1, wx.ALIGN_CENTER_VERTICAL)
 	horbox.Add((15,-1))
        horbox.Add(self.include_file_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.include_file, 2, wx.LEFT, border=-5)
        static_horbox.Add(horbox, 1, wx.EXPAND|wx.ALL, border=5)
 	verbox.Add((10,10))
        verbox.Add(static_horbox, 0, wx.EXPAND)

        static_horbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Exclude"), wx.HORIZONTAL)
        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add(self.exclude_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.exclude, 1, wx.ALIGN_CENTER_VERTICAL)
 	horbox.Add((15,-1))
        horbox.Add(self.exclude_file_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.exclude_file, 2, wx.LEFT, border=-5)
        static_horbox.Add(horbox, 1, wx.EXPAND|wx.ALL, border=5)
 	verbox.Add((10,10))
        verbox.Add(static_horbox, 0, wx.EXPAND)

        static_horbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Suppress"), wx.HORIZONTAL)
        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add(self.suppress_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.suppress, 1, wx.ALIGN_CENTER_VERTICAL)
 	horbox.Add((15,-1))
        horbox.Add(self.suppress_file_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.suppress_file, 2, wx.LEFT, border=-5)
        static_horbox.Add(horbox, 1, wx.EXPAND|wx.ALL, border=5)
 	verbox.Add((10,10))
        verbox.Add(static_horbox, 0, wx.EXPAND)

        static_horbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Tidy options"), wx.HORIZONTAL)
        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add(self.tidy_options_str_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.tidy_options_str, 1, wx.ALIGN_CENTER_VERTICAL)
 	horbox.Add((15,-1))
        horbox.Add(self.tidy_options_file_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.tidy_options_file, 2, wx.LEFT, border=-5)
        static_horbox.Add(horbox, 1, wx.EXPAND|wx.ALL, border=5)
 	verbox.Add((10,10))
        verbox.Add(static_horbox, 0, wx.EXPAND)

        static_horbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Summary report"), wx.HORIZONTAL)
        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add(self.mailto_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.mailto, 1, wx.ALIGN_CENTER_VERTICAL)
 	horbox.Add((15,-1))
        horbox.Add(self.note_label, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
        horbox.Add(self.note, 2, wx.ALIGN_CENTER_VERTICAL)
        static_horbox.Add(horbox, 1, wx.EXPAND|wx.ALL, border=5)
 	verbox.Add((10,10))
        verbox.Add(static_horbox, 0, wx.EXPAND)

        self.advanced_options_sizer.Add(verbox, 0, wx.EXPAND|wx.ALL, border=10)

        self.mainsizer.Add(self.advanced_options_sizer, 0, wx.EXPAND|wx.BOTTOM, border=10)
        self.mainsizer.Show(self.advanced_options_sizer, self.history.show_advanced_options)

        horbox = wx.BoxSizer(wx.HORIZONTAL)
        horbox.Add((10,-1), 1)
        horbox.Add(self.applyButton, 0, wx.ALIGN_RIGHT)

        self.mainsizer.Add(horbox, 0, wx.EXPAND)

        self.box = wx.BoxSizer(wx.HORIZONTAL)
        self.box.Add(self.mainsizer, 1, wx.EXPAND|wx.ALL, 10)
        self.SetSizer(self.box)


    def InstallEventListeners(self):
        self.Bind(wx.EVT_BUTTON, self.OnRun, self.applyButton)
        self.Bind(wx.EVT_CHECKBOX, self.OnShowResults, self.show_resultsCheckBox)
        self.Bind(wx.EVT_CHECKBOX, self.OnXMLPass, self.xml_passCheckBox)
        self.Bind(wx.EVT_BUTTON, self.OnAddSource, self.add_sourceButton)
        self.Bind(wx.EVT_BUTTON, self.OnDelSource, self.del_sourceButton)
        self.Bind(wx.EVT_BUTTON, self.OnClearSource, self.clear_sourceButton)

	self.mailto.Bind(wx.EVT_KILL_FOCUS, self.OnLeaveMailto)

        EVT_UPDATE(self, self.OnUpdate)
        EVT_FINISHED(self, self.OnFinished)
        EVT_EXCEPTION(self, self.OnException)


    def OnAddSource(self, event):
        dialog = wx.DirDialog(self,
                              message = "Select source directory",
                              defaultPath = get_home_dir())

        if dialog.ShowModal() == wx.ID_OK:
            self.source_directories.Append(dialog.GetPath())
        dialog.Destroy()


    def OnDelSource(self, event):
        selections = list(self.source_directories.GetSelections())
        for index in reversed(selections):
            self.source_directories.Delete(index)


    def OnClearSource(self, event):
        self.source_directories.Clear()


    def OnLeaveMailto(self, event):
	have_mailto = (self.mailto.GetValue().strip() != '')
	self.note_label.Enable(have_mailto)
	self.note.Enable(have_mailto)


    def OnShowResults(self, event):
        self.history.show_results = self.show_resultsCheckBox.GetValue()


    def OnXMLPass(self, event):
        self.history.xml_pass = self.xml_passCheckBox.GetValue()


    def ShowAdvancedOptions(self, showit):
        self.mainsizer.Show(self.advanced_options_sizer, showit)
        self.history.show_advanced_options = showit
        self.SetSizer(self.box)


    def ShowError(self, message):
        wx.MessageBox(message,
                      'Tidybot Error',
                      wx.OK | wx.ICON_ERROR, self)


    def ShowRuntimeError(self, message):
        wx.MessageBox(message,
                      'Tidybot Runtime Error',
                      wx.OK | wx.ICON_ERROR, self)


    def InitHistoryLists(self):

        self.output_directory.SetHistory(self.history.output_directories)
        if self.history.output_directory:
            self.output_directory.SetValue(self.history.output_directory)
            self.output_directory.startDirectory = os.path.dirname(self.history.output_directory)

        self.include_file.SetHistory(self.history.include_files)
        if self.history.include_file:
            self.include_file.SetValue(self.history.include_file)
            self.include_file.startDirectory = os.path.dirname(self.history.include_file)

        self.exclude_file.SetHistory(self.history.exclude_files)
        if self.history.exclude_file:
            self.exclude_file.SetValue(self.history.exclude_file)
            self.exclude_file.startDirectory = os.path.dirname(self.history.exclude_file)

        self.suppress_file.SetHistory(self.history.suppress_files)
        if self.history.suppress_files:
            self.suppress_file.SetValue(self.history.suppress_file)
            self.suppress_file.startDirectory = os.path.dirname(self.history.suppress_file)

        self.tidy_options_file.SetHistory(self.history.tidy_options_files, selectionIndex=0)
        if self.history.tidy_options_file:
            self.tidy_options_file.SetValue(self.history.tidy_options_file)
            self.tidy_options_file.startDirectory = os.path.dirname(self.history.tidy_options_file)


    def UpdateHistoryList(self, control):
        history = control.GetHistory()
        value = control.GetValue()
        if value not in history:
            history.append(value)
            control.SetHistory(history)
            control.GetHistoryControl().SetStringSelection(value)


    def UpdateHistoryLists(self):

        self.history.source_directories = []
        for index in range(self.source_directories.GetCount()):
            self.history.source_directories.append(self.source_directories.GetString(index))

        for control in self.historyControls:
            self.UpdateHistoryList(control)

        self.history.output_directories = self.output_directory.GetHistory()
        self.history.include_files = self.include_file.GetHistory()
        self.history.exclude_files = self.exclude_file.GetHistory()
        self.history.suppress_files = self.suppress_file.GetHistory()
        self.history.tidy_options_files = self.tidy_options_file.GetHistory()

        self.history.output_directory = self.output_directory.GetValue()

        self.history.include = self.include.GetValue()
        self.history.include_file = self.include_file.GetValue()

        self.history.exclude = self.exclude.GetValue()
        self.history.exclude_file = self.exclude_file.GetValue()

        self.history.suppress = self.suppress.GetValue()
        self.history.suppress_file = self.suppress_file.GetValue()

        self.history.tidy_options_str = self.tidy_options_str.GetValue()
        self.history.tidy_options_file = self.tidy_options_file.GetValue()
        self.history.stylesheet = self.stylesheet.GetValue()

        self.history.mailto = self.mailto.GetValue()
        self.history.note = self.note.GetValue()


    def OnRun(self, evt):

        source_directories = []
        for index in range(self.source_directories.GetCount()):
            source_directories.append(self.source_directories.GetString(index))
        include = self.include.GetValue()
        include_file = self.include_file.GetValue()
        exclude = self.exclude.GetValue()
        exclude_file = self.exclude_file.GetValue()
        suppress = self.suppress.GetValue()
        suppress_file = self.suppress_file.GetValue()
        tidy_options_str = self.tidy_options_str.GetValue()
        tidy_options_file = self.tidy_options_file.GetValue()
        stylesheet = self.stylesheet.GetValue()
        mailto = self.mailto.GetValue()
        note = self.note.GetValue()
        output_directory = self.output_directory.GetValue()
        xml_pass = self.xml_passCheckBox.GetValue()

        if not source_directories:
            self.ShowError('You must specify a source directory')
            return

        for dir in source_directories:
            if not os.path.exists(dir):
                self.ShowError("Source directory does not exist: %s" % dir)
                return

            if not os.path.isdir(dir):
                self.ShowError("Source string is not a directory: %s" % dir)
                return

            if '"' in dir:
                self.ShowError('Source directory may not contain quote (") characters')
                return

        if not output_directory:
            self.ShowError('You must specify a output directory')
            return

	if not include and not include_file:
            self.ShowError("No include filenames specified")
            return

        if not os.path.exists(output_directory):
            self.ShowError("Output directory does not exist: %s" % output_directory)
            return

        if not os.path.isdir(output_directory):
            self.ShowError("Output string is not a directory: %s" % output_directory)
            return

        self.busy = wx.BusyCursor()

        cmd_tokens = {}
        if output_directory:
            cmd_tokens['output_directory'] = output_directory
        if include:
            cmd_tokens['include'] = include
        if include_file:
            cmd_tokens['include_from'] = include_file
        if exclude:
            cmd_tokens['exclude'] = exclude
        if exclude_file:
            cmd_tokens['exclude_from'] = exclude_file
        if suppress:
            cmd_tokens['suppress'] = suppress
        if suppress_file:
            cmd_tokens['suppress_from'] = suppress_file
        if tidy_options_str:
            cmd_tokens['tidy_options_str'] = tidy_options_str
        if tidy_options_file:
            cmd_tokens['tidy_options_from'] = tidy_options_file
        if stylesheet:
            cmd_tokens['stylesheet'] = stylesheet
        if mailto:
            cmd_tokens['mailto'] = mailto
        if note:
            cmd_tokens['note'] = note
        if xml_pass:
            cmd_tokens['xml_pass'] = True

        try:
            config = tidybot.Config(cmd_tokens, source_directories)
	except tidybot.TidybotError, eeks:
            wx.MessageBox(str(eeks),
                          eeks.label,
                          wx.OK|wx.ICON_ERROR, self)
            del self.busy
            return

        if config.max_count > 0:
            self.progress_dialog = wx.ProgressDialog("Running Tidybot...",
                                                     "Checking files",
                                                     config.max_count,
                                                     self,
                                                     # wx.PD_CAN_ABORT|wx.PD_APP_MODAL|wx.PD_AUTO_HIDE|wx.PD_SMOOTH)
                                                     wx.PD_APP_MODAL|wx.PD_AUTO_HIDE|wx.PD_SMOOTH)
            self.progress_dialog.SetSize((300,-1))

        self.worker = WorkerThread(self, config)


    def OnUpdate(self, event):
        if self.worker:
            self.progress_dialog.Update(event.count, "Checking: %s of %s files" % (event.count, event.max_count))


    def OnException(self, event):
        if self.worker:
            self.worker.abort()
        if self.progress_dialog:
            self.progress_dialog.Destroy()
	if isinstance(event.eeks, tidybot.TidybotError):
	    self.ShowRuntimeError(str(event.eeks))
        else:
            raise event.eeks
        del self.busy


    def OnFinished(self, event):
        output_directory = self.output_directory.GetValue()
        output_file = os.path.join(output_directory, "tidybot.html")

        if self.show_resultsCheckBox.GetValue():
            try:
                tidybot_gui.wingwebbrowser.open("file://" + output_file)
            except tidybot_gui.wingwebbrowser.Error, e:
                wx.MessageBox("Error launching browser:\n%s." % str(e),
                              "Tidybot Error",
                              wx.OK|wx.ICON_ERROR, self)
                del self.busy
                if self.progress_dialog:
                    self.progress_dialog.Update(config.max_count)
                return
            wx.MessageBox("Tidybot has succesfully finished. The results have been sent to your browser.",
                          "Tidybot Message",
                          wx.OK|wx.ICON_INFORMATION, self)
        else:
            wx.MessageBox("Tidybot has succesfully finished. The results are in directory\n'%s'" %
			  output_directory,
                          "Tidybot Message",
                          wx.OK|wx.ICON_INFORMATION, self)

        self.UpdateHistoryLists()

        del self.busy


class TidybotFrame(wx.Frame):

    def __init__(self, parent, ID, title, pos, size):
        wx.Frame.__init__(self, parent, ID, title, pos, size)

        package_dir = get_main_dir()
        self.iconfile = os.path.join(package_dir, "tidybot.ico")
        ib=wx.IconBundle()
        ib.AddIconFromFile(self.iconfile,wx.BITMAP_TYPE_ANY)
        self.SetIcons(ib)

        self.history = GUI_History()
        self.history.Read()

        self.CreateControls()
        self.CreateLayout()
        self.InstallEventListeners()


    def CreateControls(self):

        self.CreateStatusBar()
        self.CreateMenuBar()
        self.panel = TidybotPanel(self, -1)
        self.SetStatusText("Welcome to Tidybot!")

    def CreateMenuBar(self):

        menuBar = wx.MenuBar()
        fileMenu = wx.Menu()
        optionsMenu = wx.Menu()
        helpMenu = wx.Menu()

        menuBar.Append(fileMenu, "&File")
        self.MENU_FILE_EXIT = fileMenu.Append(-1, "&Quit\tCtrl-Q", "Exit Tidybot")
        menuBar.Append(optionsMenu, "&Options")
        self.MENU_OPTIONS_ADVANCED = optionsMenu.AppendCheckItem(-1, "Show &Advanced Options\tCtrl-A", "Show advanced configuration options")
        menuBar.Append(helpMenu, "&Help")
        self.MENU_HELP_README = helpMenu.Append(-1, "Tidybot &Documentation\tF1", "Display the Tidybot README/Manual file")
        helpMenu.AppendSeparator()
        self.MENU_HELP_WEBSITE = helpMenu.Append(-1, "Tidybot &Website", "Display the Tidybot home page in the default browser")
        helpMenu.AppendSeparator()
        self.MENU_HELP_ABOUT = helpMenu.Append(-1, "&About Tidybot", "Display Tidybot credits and version information")

        self.SetMenuBar(menuBar)

        self.MENU_OPTIONS_ADVANCED.Check(self.history.show_advanced_options)


    def CreateLayout(self):
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.panel, 1, wx.EXPAND)

        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)
        self.Fit()


    def InstallEventListeners(self):
        self.Bind(wx.EVT_MENU, self.OnExit, self.MENU_FILE_EXIT)
        self.Bind(wx.EVT_MENU, self.OnAdvancedOptions, self.MENU_OPTIONS_ADVANCED)
        self.Bind(wx.EVT_MENU, self.OnReadme, self.MENU_HELP_README)
        self.Bind(wx.EVT_MENU, self.OnWebsite, self.MENU_HELP_WEBSITE)
        self.Bind(wx.EVT_MENU, self.OnAbout, self.MENU_HELP_ABOUT)
        self.Bind(wx.EVT_CLOSE, self.OnCloseFrame)


    def OnAdvancedOptions(self, evt):
        self.panel.ShowAdvancedOptions(evt.IsChecked())
        self.SetSizer(self.sizer)
        self.Fit()


    def OnExit(self, evt):
        self.Close(True)

    def OnAbout(self, evt):
	version = """\
Tidybot %s, %s
Written by %s
(using TidyLib release %s)
(using wxPython v%s)
"""
        version = version % (tidybot.__version__, tidybot.__date__,
                             tidybot.__author__,
                             ctypes.string_at(tidy.lib._tidy.ReleaseDate()),
                             wx.VERSION_STRING)

        dlg = wx.MessageDialog(self,
                               version,
                               'About Tidybot',
                               wx.OK | wx.ICON_INFORMATION
                               )
        dlg.ShowModal()
        dlg.Destroy()

    def OnReadme(self, evt):
        try:
            if self.readmeframe.IsIconized():
                self.readmeframe.Iconize(False)
            if not self.readmeframe.IsShown():
                self.readmeframe.Show(True)
            self.readmeframe.Raise()
        except AttributeError:
            self.readmeframe = tidybot_gui.ReadmeFrame.ReadmeFrame(self)
            self.readmeframe.Show(True)

    def OnWebsite(self, evt):
        try:
            tidybot_gui.wingwebbrowser.open("http://www.kronto.org/tidybot/")
        except tidybot_gui.wingwebbrowser.Error, e:
            wx.MessageBox("Error launching browser:\n%s." % str(e),
                          "Tidybot Error",
                          wx.OK|wx.ICON_ERROR, self)
            return
        wx.MessageBox("The Tidybot website has been opened in your browser.",
                      "Tidybot Message",
                      wx.OK|wx.ICON_INFORMATION, self)


    def OnCloseFrame(self, evt):
        evt.Skip()
        self.panel.UpdateHistoryLists()
        self.history.Write()



class TidybotApp(wx.App):

    def __init__(self, name):
        self.name = name
        wx.App.__init__(self, redirect=False)


    def OnInit(self):

	pos, size = self.PlaceWindow((640,260))
        self.frame = TidybotFrame(None, -1, self.name, pos, size)
        self.frame.Show(True)
        return True

    def PlaceWindow(self, size):
        screen_width = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_X)
        screen_height =  wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)
        width = min(size[0], screen_width);
        height = min(size[1], screen_height);
        x_min = (screen_width - width) / 2;
        y_min = (screen_height - height) / 6;
	pos = (x_min, y_min)
	size = (width, height)
	return pos, size


def main():

    mongoose.start()
    mongoose.setSessionInfo("Tidybot", tidybot.__version__)
    mongoose.setSessionInfo("Host", platform.platform())

    reporter = mongoose.reporters.GuiReporter("An internal error has occurred...")
    mongoose.registerReporter(reporter)

    appdir = tidybot_gui.homedirectory.appdatadirectory()
    if sys.platform == 'win32':
        logfile = os.path.join(appdir, "Tidybot", "tidybot-err.log")
    else:
        logfile = os.path.join(appdir, ".tidybot", "tidybot-err.log")
    reporter.MESSAGE_TEXT = "Tidybot has encountered a problem and is, alas, unable to continue.\n\nPlease consider mailing the author at <leo@kronto.org> and sending him the log file: '%s'." % logfile

    mongoose.registerReporter(mongoose.reporters.FileLogReporter(logfile))
    app = TidybotApp("Tidybot %s" % tidybot.__version__)
    app.MainLoop()
    mongoose.stop()
    return 0


if __name__ == "__main__":
    sys.exit(main())
